@@ -10,11 +10,10 @@ from pymbolic.mapper.stringifier import (
 )
 from pymbolic.primitives import FloorDiv, Remainder
 
-from loki.ir import Stringifier
+from loki.ir import Stringifier, get_pragma_parameters
 from loki.tools import as_tuple, JoinableStringList, flatten
 from loki.expression import LokiStringifyMapper, StringLiteral
 from loki.types import DataType, BasicType, DerivedType, ProcedureType
-from loki.pragma_utils import get_pragma_parameters
 
 
 __all__ = ['fgen', 'fexprgen', 'FortranCodegen', 'FCodeMapper']
@@ -210,7 +209,8 @@ class FortranCodegen(Stringifier):
         if o.prefix:
             prefix += ' '
         arguments = self.join_items(o.argnames)
-        result = f' RESULT({o.result_name})' if o.result_name else ''
+        result = f' RESULT({o.result_name})' if o.result_name\
+                and o.result_name.lower() != o.name.lower() else ''
         if isinstance(o.bind, str):
             bind_c = f' BIND(c, name="{o.bind}")'
         elif isinstance(o.bind, StringLiteral):
