@@ -116,18 +116,21 @@ class TypedSymbol:
         The type of that symbol. Defaults to :any:`BasicType.DEFERRED`.
     parent : :any:`Scalar` or :any:`Array`, optional
         The derived type variable this variable belongs to.
+    case_sensitive : bool, optional
+        Mark the name of this symbol as case-sensitive (default: `False`)
     *args : optional
         Any other positional arguments for other parent classes
     **kwargs : optional
         Any other keyword arguments for other parent classes
     """
 
-    init_arg_names = ('name', 'scope', 'parent', 'type', )
+    init_arg_names = ('name', 'scope', 'parent', 'type', 'case_sensitive', )
 
     def __init__(self, *args, **kwargs):
         self.name = kwargs['name']
         self.parent = kwargs.pop('parent', None)
         self.scope = kwargs.pop('scope', None)
+        self.case_sensitive = kwargs.pop('case_sensitive', config['case-sensitive'])
 
         # Use provided type or try to determine from scope
         self._type = None
@@ -154,7 +157,7 @@ class TypedSymbol:
         symbol objects. We do not recurse here, since we own the
         "name" attribute, which pymbolic will otherwise replicate.
         """
-        return (self.name, None, self._parent, self._type, )
+        return (self.name, None, self._parent, self._type, self.case_sensitive, )
 
     @property
     def scope(self):
@@ -339,6 +342,8 @@ class TypedSymbol:
                 kwargs['type'] = self.type
         if 'parent' not in kwargs and self.parent:
             kwargs['parent'] = self.parent
+        if 'case_sensitive' not in kwargs and self.case_sensitive:
+            kwargs['case_sensitive'] = self.case_sensitive
 
         return Variable(**kwargs)
 
@@ -357,6 +362,16 @@ class TypedSymbol:
                 return self.clone(scope=scope, type=existing_type)
         return self.clone(scope=scope)
 
+    def get_derived_type_member(self, name_str):
+        """
+        Resolve type-bound variables of arbitrary nested depth.
+        """
+        name_parts = name_str.split('%', maxsplit=1)
+        declared_var = Variable(name=f'{self.name}%{name_parts[0]}', scope=self.scope, parent=self)
+        if len(name_parts) > 1:
+            return declared_var.get_derived_type_member(name_parts[1])  # pylint:disable=no-member
+        return declared_var
+
 
 class DeferredTypeSymbol(StrCompareMixin, TypedSymbol, pmbl.Variable):  # pylint: disable=too-many-ancestors
     """
@@ -629,6 +644,20 @@ class MetaSymbol(StrCompareMixin, pmbl.AlgebraicLeaf):
         """
         return self.symbol.rescope(scope)
 
+    @property
+    def case_sensitive(self):
+        """
+        Property to indicate that the name of this symbol is case-sensitive.
+        """
+        return self.symbol.case_sensitive
+
+    def get_derived_type_member(self, name_str):
+        """
+        Resolve type-bound variables of arbitrary nested depth.
+        """
+
+        return self.symbol.get_derived_type_member(name_str)
+
 
 class Scalar(MetaSymbol):  # pylint: disable=too-many-ancestors
     """
@@ -681,7 +710,6 @@ class Array(MetaSymbol):
     def __init__(self, name, scope=None, type=None, dimensions=None, **kwargs):
         # Stop complaints about `type` in this function
         # pylint: disable=redefined-builtin
-
         symbol = VariableSymbol(name=name, scope=scope, type=type, **kwargs)
         if dimensions:
             symbol = ArraySubscript(symbol, dimensions)
