@@ -12,8 +12,12 @@ from codetiming import Timer
 
 from loki.frontend.source import Source
 from loki.frontend.util import OMNI, sanitize_ir
-from loki.ir import GenericVisitor, FindNodes, Transformer
+
 from loki import ir
+from loki.ir import (
+    GenericVisitor, FindNodes, Transformer,
+    process_dimension_pragmas, pragmas_attached
+)
 from loki.expression import (
     symbols as sym, operations as op,
     ExpressionDimensionsMapper, StringConcat, AttachScopesMapper
@@ -23,9 +27,6 @@ from loki.config import config
 from loki.tools import (
     as_tuple, execute, gettempdir, filehash, CaseInsensitiveDict
 )
-from loki.pragma_utils import (
-    process_dimension_pragmas, pragmas_attached
-)
 from loki.types import BasicType, DerivedType, ProcedureType, SymbolAttributes
 
 
@@ -432,7 +433,7 @@ class OMNI2IR(GenericVisitor):
 
         # Update array shapes with Loki dimension pragmas
         with pragmas_attached(routine, ir.VariableDeclaration):
-            routine.spec = process_dimension_pragmas(routine.spec)
+            routine.spec = process_dimension_pragmas(routine.spec, scope=routine)
 
         return routine
 
