@@ -10,9 +10,10 @@ from loki.frontend import (
     parse_omni_ast, parse_ofp_ast, parse_fparser_ast, get_fparser_node,
     parse_regex_source
 )
-from loki.ir import nodes as ir, FindNodes, Transformer
+from loki.ir import (
+    nodes as ir, FindNodes, Transformer, is_loki_pragma, pragmas_attached
+)
 from loki.logging import debug
-from loki.pragma_utils import is_loki_pragma, pragmas_attached
 from loki.program_unit import ProgramUnit
 from loki.tools import as_tuple, CaseInsensitiveDict
 from loki.types import BasicType, ProcedureType, SymbolAttributes
@@ -104,6 +105,10 @@ class Subroutine(ProgramUnit):
         self.result_name = result_name
         self.is_function = is_function
 
+        # Make sure 'result_name' is defined if it's a function
+        if self.result_name is None and self.is_function:
+            self.result_name = name
+
         # Additional IR components
         if body is not None and not isinstance(body, ir.Section):
             body = ir.Section(body=body)
@@ -327,9 +332,7 @@ class Subroutine(ProgramUnit):
         """
         if not self.is_function:
             return None
-        if self.result_name is not None:
-            return self.symbol_attrs.get(self.result_name)
-        return self.symbol_attrs.get(self.name)
+        return self.symbol_attrs.get(self.result_name)
 
     variables = ProgramUnit.variables
 
