@@ -8,8 +8,7 @@
 from pathlib import Path
 
 from loki.backend import fgen
-from loki.batch import SchedulerConfig
-from loki.expression import Variable, FindInlineCalls, SubstituteExpressions
+from loki.expression import Variable, FindInlineCalls
 from loki.ir import (
     CallStatement, Import, Section, Interface, FindNodes, Transformer
 )
@@ -156,8 +155,6 @@ class DependencyTransformation(Transformation):
                 return
 
             # Change the name of kernel routines
-            if routine.is_function and not routine.result_name:
-                self.update_result_var(routine)
             routine.name += self.suffix
             if item:
                 item.name += self.suffix.lower()
@@ -221,24 +218,6 @@ class DependencyTransformation(Transformation):
             return f'{modname}{self.suffix}{self.module_suffix}'
         return f'{modname}{self.suffix}'
 
-    def update_result_var(self, routine):
-        """
-        Update name of result variable for function calls.
-
-        Parameters
-        ----------
-        routine : :any:`Subroutine`
-            The function object for which the result variable is to be renamed
-        """
-        assert routine.name in routine.variables
-
-        vmap = {
-            v: v.clone(name=v.name + self.suffix)
-            for v in routine.variables if v == routine.name
-        }
-        routine.spec = SubstituteExpressions(vmap).visit(routine.spec)
-        routine.body = SubstituteExpressions(vmap).visit(routine.body)
-
     def rename_calls(self, routine, targets=None, item=None):
         """
         Update :any:`CallStatement` and :any:`InlineCall` to actively
@@ -250,6 +229,8 @@ class DependencyTransformation(Transformation):
             Optional list of subroutine names for which to modify the corresponding
             calls. If not provided, all calls are updated
         """
+        from loki.batch import SchedulerConfig  # pylint: disable=import-outside-toplevel,cyclic-import
+
         def _update_item(orig_name, new_name):
             # Update the ignore property if necessary
             if item and (matched_keys := SchedulerConfig.match_item_keys(orig_name, item.ignore)):
@@ -318,8 +299,8 @@ class DependencyTransformation(Transformation):
         import_map = {}
         for im in imports:
             if im.c_import:
-                target_symbol = im.module.split('.')[0].lower()
-                if targets and target_symbol.lower() in targets:
+                target_symbol, *suffixes = im.module.lower().split('.', maxsplit=1)
+                if targets and target_symbol.lower() in targets and not 'func.h' in suffixes:
                     # Modify the the basename of the C-style header import
                     s = '.'.join(im.module.split('.')[1:])
                     im._update(module=f'{target_symbol}{self.suffix}.{s}')
@@ -468,6 +449,8 @@ class ModuleWrapTransformation(Transformation):
         """
         Update imports of wrapped subroutines.
         """
+        from loki.batch import SchedulerConfig  # pylint: disable=import-outside-toplevel,cyclic-import
+
         targets = tuple(str(t).lower() for t in as_tuple(kwargs.get('targets')))
         if self.replace_ignore_items and (item := kwargs.get('item')):
             targets += tuple(str(i).lower() for i in item.ignore)
@@ -486,8 +469,8 @@ class ModuleWrapTransformation(Transformation):
         # We go through the IR, as C-imports can be attributed to the body
         for im in imports:
             if im.c_import:
-                target_symbol = im.module.split('.')[0].lower()
-                if targets and target_symbol.lower() in targets:
+                target_symbol, *suffixes = im.module.lower().split('.', maxsplit=1)
+                if targets and target_symbol.lower() in targets and not 'func.h' in suffixes:
                     # Create a new module import with explicitly qualified symbol
                     modname = f'{target_symbol}{self.module_suffix}'
                     _update_item(target_symbol.lower(), modname)
