@@ -87,7 +87,9 @@ from loki.tools.util import is_iterable, as_tuple, CaseInsensitiveDict
 from loki.transform.transformation import Transformation
 from loki.transform.transform_utilities import single_variable_declaration
 from loki.batch.item import ProcedureItem
+from loki.expression.expr_visitors import FindInlineCalls
 import loki.expression.symbols as sym
+from loki.expression.symbolic import is_dimension_constant
 
 
 __all__ = ['HoistVariablesAnalysis', 'HoistVariablesTransformation',
@@ -101,12 +103,6 @@ class HoistVariablesAnalysis(Transformation):
     Traverses all subroutines to find the variables to be hoisted.
     Create a derived class and override :func:`find_variables<HoistVariablesAnalysis.find_variables>`
     to define which variables to be hoisted.
-
-    Parameters
-    ----------
-    key : str
-        Access identifier/key for the ``item.trafo_data`` dictionary. Only necessary to provide if several of
-        these transformations are carried out in succession.
     """
 
     _key = 'HoistVariablesTransformation'
@@ -116,10 +112,6 @@ class HoistVariablesAnalysis(Transformation):
 
     process_ignored_items = True
 
-    def __init__(self, key=None):
-        if key is not None:
-            self._key = key
-
     def transform_subroutine(self, routine, **kwargs):
         """
         Analysis applied to :any:`Subroutine` item.
@@ -151,6 +143,7 @@ class HoistVariablesAnalysis(Transformation):
             item.trafo_data[self._key]["hoist_variables"] = []
 
         calls = FindNodes(CallStatement).visit(routine.body)
+        calls += FindInlineCalls().visit(routine.body)
         call_map = CaseInsensitiveDict((str(call.name), call) for call in calls)
 
         for child in successors:
@@ -197,18 +190,13 @@ class HoistVariablesTransformation(Transformation):
 
     Parameters
     ----------
-    key : str
-        Access identifier/key for the ``item.trafo_data`` dictionary. Only necessary to provide if several of
-        these transformations are carried out in succession.
     as_kwarguments : boolean
         Whether to pass the hoisted arguments as `args` or `kwargs`.
     """
 
     _key = 'HoistVariablesTransformation'
 
-    def __init__(self, key=None, as_kwarguments=False):
-        if key is not None:
-            self._key = key
+    def __init__(self, as_kwarguments=False):
         self.as_kwarguments = as_kwarguments
 
     def transform_subroutine(self, routine, **kwargs):
@@ -255,7 +243,7 @@ class HoistVariablesTransformation(Transformation):
             routine.arguments += hoisted_temporaries
 
         call_map = {}
-        for call in FindNodes(CallStatement).visit(routine.body):
+        for call in FindNodes(CallStatement).visit(routine.body) + list(FindInlineCalls().visit(routine.body)):
             # Only process calls in this call tree
             if str(call.name) not in successor_map:
                 continue
@@ -272,9 +260,14 @@ class HoistVariablesTransformation(Transformation):
                     routine=routine, call=call, variables=hoisted_variables
                 )
             elif role == "kernel":
-                call_map[call] = self.kernel_call_argument_remapping(
-                    routine=routine, call=call, variables=hoisted_variables
-                )
+                if isinstance(call, CallStatement):
+                    call_map[call] = self.kernel_call_argument_remapping(
+                        routine=routine, call=call, variables=hoisted_variables
+                    )
+                else:
+                    self.kernel_inline_call_argument_remapping(
+                        routine=routine, call=call, variables=hoisted_variables
+                    )
 
         routine.body = Transformer(call_map).visit(routine.body)
 
@@ -356,6 +349,30 @@ class HoistVariablesTransformation(Transformation):
         new_args = tuple(v.clone(dimensions=None) for v in variables)
         return call.clone(arguments=call.arguments + new_args)
 
+    def kernel_inline_call_argument_remapping(self, routine, call, variables):
+        """
+        Append hoisted temporaries to inline function call arguments.
+
+        Parameters
+        ----------
+        routine : :any:`Subroutine`
+            The subroutine to add the variable declaration to.
+        call : :any:`InlineCall`
+            ProcedureSymbol to which hoisted variables will be added.
+        variables : tuple of :any:`Variable`
+            The tuple of variables to be declared.
+        """
+
+        if self.as_kwarguments:
+            kw_params = call.kw_parameters
+            kw_params.update(dict((a.name, v.clone(dimensions=None)) for (a, v) in variables))
+            _call_clone = call.clone(kw_parameters=kw_params)
+            vmap = {call: _call_clone}
+        else:
+            new_args = tuple(v.clone(dimensions=None) for v in variables)
+            vmap = {call: call.clone(parameters=call.parameters + new_args)}
+
+        routine.body = SubstituteExpressions(vmap).visit(routine.body)
 
 class HoistTemporaryArraysAnalysis(HoistVariablesAnalysis):
     """
@@ -371,19 +388,16 @@ class HoistTemporaryArraysAnalysis(HoistVariablesAnalysis):
 
     Parameters
     ----------
-    key : str, optional
-        Access identifier/key for the ``item.trafo_data`` dictionary. Only necessary to provide if several of
-        these transformations are carried out in succession.
     dim_vars: tuple of str, optional
-        Variables to be within the dimensions of the arrays to be hoisted. If not provided, no checks will be done
-        for the array dimensions.
+        Variables to be within the dimensions of the arrays to be
+        hoisted. If not provided, no checks will be done for the array
+        dimensions.
     """
 
     # Apply in reverse order to recursively find all variables to be hoisted.
     reverse_traversal = True
 
-    def __init__(self, key=None, dim_vars=None, **kwargs):
-        super().__init__(key=key, **kwargs)
+    def __init__(self, dim_vars=None):
         self.dim_vars = dim_vars
         if self.dim_vars is not None:
             assert is_iterable(self.dim_vars)
@@ -400,10 +414,16 @@ class HoistTemporaryArraysAnalysis(HoistVariablesAnalysis):
         routine : :any:`Subroutine`
             The subroutine find the variables.
         """
-        return [var for var in routine.variables
+
+        # Determine function result variable name
+        if not (result_name := routine.result_name):
+            result_name = routine.name
+
+        variables = [var for var in routine.variables if isinstance(var, sym.Array)]
+        return [var for var in variables
                 if var not in routine.arguments    # local variable
-                and not var.type.parameter         # not a parameter
-                and isinstance(var, sym.Array)     # is an array
+                and not all(is_dimension_constant(d) for d in var.shape)
+                and not var.name.lower() == result_name.lower()
                 and (self.dim_vars is None         # if dim_vars not empty check if at least one dim is within dim_vars
                      or any(dim_var in self.dim_vars for dim_var in FindVariables().visit(var.dimensions)))]
 
@@ -414,17 +434,8 @@ class HoistTemporaryArraysTransformationAllocatable(HoistVariablesTransformation
     functionality/transformation, to hoist temporary arrays and make
     them ``allocatable``, including the actual *allocation* and
     *de-allocation*.
-
-    Parameters
-    ----------
-    key : str, optional
-        Access identifier/key for the ``item.trafo_data`` dictionary. Only necessary to provide if several of
-        these transformations are carried out in succession.
     """
 
-    def __init__(self, key=None, **kwargs):
-        super().__init__(key=key, **kwargs)
-
     def driver_variable_declaration(self, routine, variables):
         """
         Declares hoisted arrays as ``allocatable``, including *allocation* and *de-allocation*.
