@@ -18,18 +18,17 @@ from loki.expression import (
 )
 from loki.ir import (
     Import, Comment, Assignment, VariableDeclaration, CallStatement,
-    Transformer, FindNodes
+    Transformer, FindNodes, pragmas_attached, is_loki_pragma
 )
 from loki.expression import symbols as sym
 from loki.types import BasicType
 from loki.tools import as_tuple
 from loki.logging import warning, error
-from loki.pragma_utils import pragmas_attached, is_loki_pragma
 from loki.subroutine import Subroutine
 
 from loki.transform.transform_sanitise import transform_sequence_association_append_map
 from loki.transform.transformation import Transformation
-from loki.transform.transform_dead_code import dead_code_elimination
+from loki.transform.transform_remove_code import do_remove_dead_code
 from loki.transform.transform_utilities import (
     single_variable_declaration,
     recursive_expression_map_update
@@ -62,7 +61,7 @@ class InlineTransformation(Transformation):
     inline_marked : bool
         Inline :any:`Subroutine` objects marked by pragma annotations
         (see :any:`inline_marked_subroutines`); default: True.
-    eliminate_dead_code : bool
+    remove_dead_code : bool
         Perform dead code elimination, where unreachable branches are
         trimmed from the code (see :any:`dead_code_elimination`); default: True
     allowed_aliases : tuple or list of str or :any:`Expression`, optional
@@ -83,7 +82,7 @@ class InlineTransformation(Transformation):
     def __init__(
             self, inline_constants=False, inline_elementals=True,
             inline_internals=False, inline_marked=True,
-            eliminate_dead_code=True, allowed_aliases=None,
+            remove_dead_code=True, allowed_aliases=None,
             remove_imports=True, external_only=True,
             resolve_sequence_association=False
     ):
@@ -91,7 +90,7 @@ class InlineTransformation(Transformation):
         self.inline_elementals = inline_elementals
         self.inline_internals = inline_internals
         self.inline_marked = inline_marked
-        self.eliminate_dead_code = eliminate_dead_code
+        self.remove_dead_code = remove_dead_code
         self.allowed_aliases = allowed_aliases
         self.remove_imports = remove_imports
         self.external_only = external_only
@@ -129,8 +128,8 @@ class InlineTransformation(Transformation):
             )
 
         # After inlining, attempt to trim unreachable code paths
-        if self.eliminate_dead_code:
-            dead_code_elimination(routine)
+        if self.remove_dead_code:
+            do_remove_dead_code(routine)
 
 
 class InlineSubstitutionMapper(LokiIdentityMapper):
@@ -498,6 +497,10 @@ def inline_subroutine_calls(routine, calls, callee, allowed_aliases=None):
     # Replace calls to child procedure with the child's body
     routine.body = Transformer(call_map).visit(routine.body)
 
+    # We need this to ensure that symbols, as well as nested scopes
+    # are correctly attached to each other (eg. nested associates).
+    routine.rescope_symbols()
+
 
 def inline_internal_procedures(routine, allowed_aliases=None):
     """
