@@ -28,7 +28,7 @@ from loki.types import SymbolAttributes, BasicType, DerivedType, ProcedureType
 
 
 __all__ = [
-    'convert_to_lower_case', 'replace_intrinsics', 'sanitise_imports',
+    'convert_to_lower_case', 'replace_intrinsics', 'rename_variables', 'sanitise_imports',
     'replace_selected_kind', 'single_variable_declaration', 'recursive_expression_map_update'
 ]
 
@@ -89,7 +89,8 @@ def convert_to_lower_case(routine):
     variables = FindVariables(unique=False).visit(routine.ir)
     vmap = {
         v: v.clone(name=v.name.lower()) for v in variables
-        if isinstance(v, (sym.Scalar, sym.Array, sym.DeferredTypeSymbol)) and not v.name.islower()
+        if isinstance(v, (sym.Scalar, sym.Array, sym.DeferredTypeSymbol)) and not v.name.islower()\
+                and not v.case_sensitive
     }
 
     # Capture nesting by applying map to itself before applying to the routine
@@ -100,7 +101,7 @@ def convert_to_lower_case(routine):
     # Downcase inline calls to, but only after the above has been propagated,
     # so that we  capture the updates from the variable update in the arguments
     mapper = {
-        c: c.clone(function=c.function.clone(name=c.name.lower()))
+        c: c.clone(function=c.function.clone(name=c.name.lower() if not c.function.case_sensitive else c.name))
         for c in FindInlineCalls().visit(routine.ir) if not c.name.islower()
     }
     mapper.update(
@@ -132,7 +133,7 @@ def replace_intrinsics(routine, function_map=None, symbol_map=None, case_sensiti
     if not case_sensitive:
         symbol_map = CaseInsensitiveDict(symbol_map)
         function_map = CaseInsensitiveDict(function_map)
-
+    # (intrinsic) functions
     callmap = {}
     for call in FindInlineCalls(unique=False).visit(routine.ir):
         if call.name in symbol_map:
@@ -144,6 +145,57 @@ def replace_intrinsics(routine, function_map=None, symbol_map=None, case_sensiti
     routine.spec = SubstituteExpressions(callmap).visit(routine.spec)
     routine.body = SubstituteExpressions(callmap).visit(routine.body)
 
+def rename_variables(routine, symbol_map=None):
+    """
+    Rename symbols/variables including (routine) arguments.
+
+    Parameters
+    ----------
+    routine : :any:`Subroutine`
+        The subroutine object in which to rename variables.
+    symbol_map : dict[str, str]
+        Mapping from symbol/variable names to their replacement.
+    """
+    symbol_map = CaseInsensitiveDict(symbol_map) or {}
+    # rename arguments if necessary
+    arguments = ()
+    renamed_arguments = ()
+    for arg in routine.arguments:
+        if arg.name in symbol_map:
+            arguments += (arg.clone(name=symbol_map[arg.name]),)
+            renamed_arguments += (arg,)
+        else:
+            arguments += (arg,)
+    routine.arguments = arguments
+    # remove variable declarations
+    var_decls = FindNodes(VariableDeclaration).visit(routine.spec)
+    var_decl_map = {}
+    for var_decl in var_decls:
+        new_symbols = ()
+        for symbol in var_decl.symbols:
+            if symbol not in renamed_arguments:
+                new_symbols += (symbol,)
+        if new_symbols:
+            var_decl_map[var_decl] = var_decl.clone(symbols=new_symbols)
+        else:
+            var_decl_map[var_decl] = None
+    routine.spec = Transformer(var_decl_map).visit(routine.spec)
+    # rename variable declarations and usages
+    var_map = {}
+    for var in FindVariables(unique=False).visit(routine.ir):
+        if var.name in symbol_map:
+            new_var = symbol_map[var.name]
+            if new_var is not None:
+                var_map[var] = var.clone(name=symbol_map[var.name])
+    if var_map:
+        routine.spec = SubstituteExpressions(var_map).visit(routine.spec)
+        routine.body = SubstituteExpressions(var_map).visit(routine.body)
+    # update symbol table - remove entries under the previous name
+    var_map_names = [key.name.lower() for key in var_map]
+    delete = [key for key in routine.symbol_attrs if key.lower() in var_map_names\
+            or key.split('%')[0].lower() in var_map_names] # derived types
+    for key in delete:
+        del routine.symbol_attrs[key]
 
 def used_names_from_symbol(symbol, modifier=str.lower):
     """
