@@ -24,24 +24,22 @@ from loki import (
 from loki.transform import (
     DependencyTransformation, ModuleWrapTransformation, FortranCTransformation,
     FileWriteTransformation, HoistTemporaryArraysAnalysis, normalize_range_indexing,
-    InlineTransformation, SanitiseTransformation
+    InlineTransformation, SanitiseTransformation, RemoveCodeTransformation, Pipeline
 )
 
 # pylint: disable=wrong-import-order
-from transformations.argument_shape import ArgumentArrayShapeAnalysis, ExplicitArgumentArrayShapeTransformation
+from transformations.argument_shape import (
+    ArgumentArrayShapeAnalysis, ExplicitArgumentArrayShapeTransformation
+)
 from transformations.data_offload import (
     DataOffloadTransformation, GlobalVariableAnalysis, GlobalVarOffloadTransformation
 )
 from transformations.derived_types import DerivedTypeArgumentsTransformation
-from transformations.utility_routines import DrHookTransformation, RemoveCallsTransformation
+from transformations.drhook import DrHookTransformation
 from transformations.pool_allocator import TemporariesPoolAllocatorTransformation
 from transformations.single_column_claw import ExtractSCATransformation, CLAWTransformation
 from transformations.single_column_coalesced import (
-    SCCBaseTransformation, SCCAnnotateTransformation,
-    SCCHoistTemporaryArraysTransformation
-)
-from transformations.single_column_coalesced_vector import (
-    SCCDevectorTransformation, SCCRevectorTransformation, SCCDemoteTransformation
+    SCCVectorPipeline, SCCHoistPipeline, SCCStackPipeline
 )
 from transformations.scc_cuf import (
     HoistTemporaryArraysDeviceAllocatableTransformation
@@ -175,6 +173,21 @@ def convert(
         paths=paths, config=config, frontend=frontend, definitions=definitions, **build_args
     )
 
+    # If requested, apply a custom pipeline from the scheduler config
+    # Note that this new entry point will bypass all other default
+    # behaviour and exit immediately after.
+    if mode in config.pipelines:
+        info(f'[Loki-transform] Applying custom pipeline {mode} from config:')
+        info(str(config.pipelines[mode]))
+
+        scheduler.process( config.pipelines[mode] )
+
+        # Write out all modified source files into the build directory
+        file_write_trafo = FileWriteTransformation(builddir=build, mode=mode)
+        scheduler.process(transformation=file_write_trafo)
+
+        return
+
     # Pull dimension definition from configuration
     horizontal = scheduler.config.dimensions.get('horizontal', None)
     vertical = scheduler.config.dimensions.get('vertical', None)
@@ -184,15 +197,20 @@ def convert(
     if remove_derived_args:
         scheduler.process( DerivedTypeArgumentsTransformation() )
 
-    # Remove DR_HOOK and other utility calls first, so they don't interfere with SCC loop hoisting
-    if 'scc' in mode:
-        scheduler.process( RemoveCallsTransformation(
-            routines=config.default.get('utility_routines', None) or ['DR_HOOK', 'ABOR1', 'WRITE(NULOUT'],
-            include_intrinsics=True, kernel_only=True
-        ))
-    else:
+    # Re-write DR_HOOK labels for non-GPU paths
+    if 'scc' not in mode:
         scheduler.process( DrHookTransformation(mode=mode, remove=False) )
 
+    # Perform general source removal of unwanted calls or code regions
+    # (do not perfrom Dead Code Elimination yet, inlining will do this.)
+    remove_code_trafo = scheduler.config.transformations.get('RemoveCodeTransformation', None)
+    if not remove_code_trafo:
+        remove_code_trafo = RemoveCodeTransformation(
+            remove_marked_regions=True, remove_dead_code=False,
+            call_names=('ABOR1', 'DR_HOOK'), intrinsic_names=('WRITE(NULOUT',)
+        )
+    scheduler.process(transformation=remove_code_trafo)
+
     # Perform general source sanitisation steps to level the playing field
     sanitise_trafo = scheduler.config.transformations.get('SanitiseTransformation', None)
     if not sanitise_trafo:
@@ -206,7 +224,7 @@ def convert(
     if not inline_trafo:
         inline_trafo = InlineTransformation(
             inline_internals=inline_members, inline_marked=inline_marked,
-            eliminate_dead_code=eliminate_dead_code, allowed_aliases=horizontal.index,
+            remove_dead_code=eliminate_dead_code, allowed_aliases=horizontal.index,
             resolve_sequence_association=resolve_sequence_association_inlined_calls 
         )
     scheduler.process(transformation=inline_trafo)
@@ -225,65 +243,71 @@ def convert(
         scheduler.process(offload_transform)
         use_claw_offload = not offload_transform.has_data_regions
 
-    # Now we instantiate our transformation pipeline and apply the main changes
-    transformation = None
-    if mode in ['idem', 'idem-stack']:
-        scheduler.process( IdemTransformation() )
+    if frontend == Frontend.OMNI and mode in ['idem-stack', 'scc-stack']:
+        # To make the pool allocator size derivation work correctly, we need
+        # to normalize the 1:end-style index ranges that OMNI introduces
+        class NormalizeRangeIndexingTransformation(Transformation):
+            def transform_subroutine(self, routine, **kwargs):
+                normalize_range_indexing(routine)
 
-    if mode == 'sca':
-        scheduler.process( ExtractSCATransformation(horizontal=horizontal) )
-
-    if mode == 'claw':
-        scheduler.process( CLAWTransformation(
-            horizontal=horizontal, claw_data_offload=use_claw_offload
-        ))
-
-    if mode in ['scc', 'scc-hoist', 'scc-stack']:
-        # Apply the basic SCC transformation set
-        scheduler.process( SCCBaseTransformation(
-            horizontal=horizontal, directive=directive
-        ))
-        scheduler.process( SCCDevectorTransformation(
-            horizontal=horizontal, trim_vector_sections=trim_vector_sections
-        ))
-        scheduler.process( SCCDemoteTransformation(horizontal=horizontal))
-        scheduler.process( SCCRevectorTransformation(horizontal=horizontal))
-
-    if mode == 'scc-hoist':
-        # Apply recursive hoisting of local temporary arrays.
-        # This requires a first analysis pass to run in reverse
-        # direction through the call graph to gather temporary arrays.
-        scheduler.process( HoistTemporaryArraysAnalysis(dim_vars=(vertical.size,)) )
-        scheduler.process( SCCHoistTemporaryArraysTransformation(block_dim=block_dim) )
-
-    if mode in ['scc', 'scc-hoist', 'scc-stack']:
-        scheduler.process( SCCAnnotateTransformation(
-                horizontal=horizontal, vertical=vertical, directive=directive, block_dim=block_dim
-        ))
-
-    if mode in ['cuf-parametrise', 'cuf-hoist', 'cuf-dynamic']:
-        # These transformations requires complex constructor arguments,
-        # so we use the file-based transformation configuration.
-        scheduler.process( transformation=scheduler.config.transformations[mode] )
+        scheduler.process( NormalizeRangeIndexingTransformation() )
 
     if global_var_offload:
         scheduler.process(transformation=GlobalVariableAnalysis())
         scheduler.process(transformation=GlobalVarOffloadTransformation())
 
-    if mode in ['idem-stack', 'scc-stack']:
-        if frontend == Frontend.OMNI:
-            # To make the pool allocator size derivation work correctly, we need
-            # to normalize the 1:end-style index ranges that OMNI introduces
-            class NormalizeRangeIndexingTransformation(Transformation):
-                def transform_subroutine(self, routine, **kwargs):
-                    normalize_range_indexing(routine)
+    # Now we create and apply the main transformation pipeline
+    if mode == 'idem':
+        pipeline = IdemTransformation()
+        scheduler.process( pipeline )
 
-            scheduler.process( NormalizeRangeIndexingTransformation() )
+    if mode == 'idem-stack':
+        pipeline = Pipeline(
+            classes=(IdemTransformation, TemporariesPoolAllocatorTransformation),
+            block_dim=block_dim, directive='openmp', check_bounds=True
+        )
+        scheduler.process( pipeline )
 
-        directive = {'idem-stack': 'openmp', 'scc-stack': 'openacc'}[mode]
-        scheduler.process(transformation=TemporariesPoolAllocatorTransformation(
-            block_dim=block_dim, directive=directive, check_bounds='scc' not in mode
-        ))
+    if mode == 'sca':
+        pipeline = ExtractSCATransformation(horizontal=horizontal)
+        scheduler.process( pipeline )
+
+    if mode == 'claw':
+        pipeline = CLAWTransformation(
+            horizontal=horizontal, claw_data_offload=use_claw_offload
+        )
+        scheduler.process( pipeline )
+
+    if mode == 'scc':
+        pipeline = SCCVectorPipeline(
+            horizontal=horizontal,
+            block_dim=block_dim, directive=directive,
+            trim_vector_sections=trim_vector_sections
+        )
+        scheduler.process( pipeline )
+
+    if mode == 'scc-hoist':
+        pipeline = SCCHoistPipeline(
+            horizontal=horizontal,
+            block_dim=block_dim, directive=directive,
+            dim_vars=(vertical.size,) if vertical else None,
+            trim_vector_sections=trim_vector_sections
+        )
+        scheduler.process( pipeline )
+
+    if mode == 'scc-stack':
+        pipeline = SCCStackPipeline(
+            horizontal=horizontal,
+            block_dim=block_dim, directive=directive,
+            check_bounds=False,
+            trim_vector_sections=trim_vector_sections )
+        scheduler.process( pipeline )
+
+
+    if mode in ['cuf-parametrise', 'cuf-hoist', 'cuf-dynamic']:
+        # These transformations requires complex constructor arguments,
+        # so we use the file-based transformation configuration.
+        scheduler.process( transformation=scheduler.config.transformations[mode] )
 
     if mode == 'cuf-parametrise':
         # This transformation requires complex constructora arguments,
@@ -319,7 +343,7 @@ def convert(
 
 @cli.command('plan')
 @click.option('--mode', '-m', default='sca',
-              type=click.Choice(['idem', 'sca', 'claw', 'scc', 'scc-hoist', 'scc-stack']))
+              type=click.Choice(['idem', 'idem-stack', 'sca', 'claw', 'scc', 'scc-hoist', 'scc-stack']))
 @click.option('--config', '-c', type=click.Path(),
               help='Path to configuration file.')
 @click.option('--header', '-I', type=click.Path(), multiple=True,
