@@ -388,24 +388,20 @@ def test_global_variable_analysis(frontend, key, config, global_variable_analysi
         nval_dim = '1:5'
         nfld_data = set()
         nval_data = set()
-        nval_offload = set()
-        nfld_offload = set()
     else:
         nfld_dim = 'nfld'
         nval_dim = 'nval'
         nfld_data = {('nfld', 'global_var_analysis_header_mod')}
         nval_data = {('nval', 'global_var_analysis_header_mod')}
-        nval_offload = {'nval'}
-        nfld_offload = {'nfld'}
 
     expected_trafo_data = {
         'global_var_analysis_header_mod': {
             'declares': {f'iarr({nfld_dim})', f'rarr({nval_dim}, {nfld_dim})'},
-            'offload': {f'iarr({nfld_dim})', f'rarr({nval_dim}, {nfld_dim})'} | nval_offload | nfld_offload,
+            'offload': {}
         },
         'global_var_analysis_data_mod': {
             'declares': {'rdata(:, :, :)', 'tt'},
-            'offload': {'rdata(:, :, :)', 'tt', 'tt%vals'}
+            'offload': {}
         },
         'global_var_analysis_data_mod#some_routine': {'defines_symbols': set(), 'uses_symbols': set()},
         'global_var_analysis_kernel_mod#kernel_a': {
@@ -454,6 +450,14 @@ def test_global_variable_offload(frontend, key, config, global_variable_analysis
         'driver': {'role': 'driver'}
     }
 
+    # OMNI handles array indices and parameters differently
+    if frontend == OMNI:
+        nfld_dim = '1:3'
+        nval_dim = '1:5'
+    else:
+        nfld_dim = 'nfld'
+        nval_dim = 'nval'
+
     scheduler = Scheduler(
         paths=(global_variable_analysis_code,), config=config, seed_routines='driver',
         frontend=frontend, xmods=(global_variable_analysis_code,)
@@ -462,6 +466,30 @@ def test_global_variable_offload(frontend, key, config, global_variable_analysis
     scheduler.process(GlobalVarOffloadTransformation(key=key))
     driver = scheduler['#driver'].ir
 
+    if key is None:
+        key = GlobalVariableAnalysis._key
+
+    expected_trafo_data = {
+        'global_var_analysis_header_mod': {
+            'declares': {f'iarr({nfld_dim})', f'rarr({nval_dim}, {nfld_dim})'},
+            'offload': {f'iarr({nfld_dim})', f'rarr({nval_dim}, {nfld_dim})'}
+        },
+        'global_var_analysis_data_mod': {
+            'declares': {'rdata(:, :, :)', 'tt'},
+            'offload': {'rdata(:, :, :)', 'tt', 'tt%vals'}
+        },
+    }
+
+    # Verify module offload sets
+    for item in [scheduler['global_var_analysis_header_mod'], scheduler['global_var_analysis_data_mod']]:
+        for trafo_data_key, trafo_data_value in item.trafo_data[key].items():
+            assert (
+                sorted(
+                    tuple(str(vv) for vv in v) if isinstance(v, tuple) else str(v)
+                    for v in trafo_data_value
+                ) == sorted(expected_trafo_data[item.name][trafo_data_key])
+            )
+
     # Verify imports have been added to the driver
     expected_imports = {
         'global_var_analysis_header_mod': {'iarr', 'rarr'},
@@ -692,6 +720,7 @@ def test_transformation_global_var_hoist(here, config, frontend, hoist_parameter
     assert [arg.name for arg in kernel0.variables] == sorted(expected_vars)
     for var in kernel0.arguments:
         assert kernel0.variable_map[var.name.lower()].type.intent == var_intent_map[var.name.lower()]
+        assert var.scope == kernel0
     kernel0_inline_calls = FindInlineCalls().visit(kernel0.body)
     for inline_call in kernel0_inline_calls:
         if ignore_modules is None:
@@ -724,6 +753,7 @@ def test_transformation_global_var_hoist(here, config, frontend, hoist_parameter
         assert [arg.name for arg in kernel_map[call.routine.name].arguments] == expected_args
         for var in kernel_map[call.routine.name].variables:
             var_intent = kernel_map[call.routine.name].variable_map[var.name.lower()].type.intent
+            assert var.scope == kernel_map[call.routine.name]
             assert var_intent == var_intent_map[var.name.lower()]
         if call.routine.name in ['kernel1', 'kernel2']:
             expected_args = ['tmp'] + expected_args
