@@ -11,16 +11,17 @@ import pytest
 
 from loki import (
     OMNI, OFP, Subroutine, Dimension, FindNodes, Loop, Assignment,
-    CallStatement, Conditional, Scalar, Array, Pragma, pragmas_attached,
-    fgen, Sourcefile, Section, ProcedureItem, ModuleItem, pragma_regions_attached, PragmaRegion,
-    is_loki_pragma, IntLiteral, RangeIndex, Comment, HoistTemporaryArraysAnalysis,
-    gettempdir, Scheduler, SchedulerConfig, SanitiseTransformation, InlineTransformation
+    CallStatement, Conditional, Scalar, Array, Pragma,
+    pragmas_attached, fgen, Sourcefile, Section, ProcedureItem,
+    ModuleItem, pragma_regions_attached, PragmaRegion, is_loki_pragma,
+    IntLiteral, RangeIndex, Comment, gettempdir, Scheduler,
+    SchedulerConfig, SanitiseTransformation, InlineTransformation
 )
 from conftest import available_frontends
 from transformations import (
     DataOffloadTransformation, SCCBaseTransformation, SCCDevectorTransformation,
     SCCDemoteTransformation, SCCRevectorTransformation, SCCAnnotateTransformation,
-    SCCHoistTemporaryArraysTransformation
+    SCCVectorPipeline, SCCHoistPipeline
 )
 #pylint: disable=too-many-lines
 
@@ -28,10 +29,14 @@ from transformations import (
 def fixture_horizontal():
     return Dimension(name='horizontal', size='nlon', index='jl', bounds=('start', 'end'), aliases=('nproma',))
 
+@pytest.fixture(scope='module', name='horizontal_bounds_aliases')
+def fixture_horizontal_bounds_aliases():
+    return Dimension(name='horizontal_bounds_aliases', size='nlon', index='jl', bounds=('start', 'end'),
+                     aliases=('nproma',), bounds_aliases=('bnds%start', 'bnds%end'))
 
 @pytest.fixture(scope='module', name='vertical')
 def fixture_vertical():
-    return Dimension(name='vertical', size='nz', index='jk')
+    return Dimension(name='vertical', size='nz', index='jk', aliases=('nlev',))
 
 
 @pytest.fixture(scope='module', name='blocking')
@@ -124,6 +129,107 @@ def test_scc_revector_transformation(frontend, horizontal):
     assert kernel_calls[0].name == 'compute_column'
 
 
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_scc_revector_transformation_aliased_bounds(frontend, horizontal_bounds_aliases):
+    """
+    Test removal of vector loops in kernel and re-insertion of a single
+    hoisted horizontal loop in the kernel with aliased loop bounds.
+    """
+
+    fcode_bnds_type_mod = """
+    module bnds_type_mod
+    implicit none
+       type bnds_type
+          integer :: start
+          integer :: end
+       end type bnds_type
+    end module bnds_type_mod
+"""
+
+    fcode_driver = """
+  SUBROUTINE column_driver(nlon, nz, q, t, nb)
+    USE bnds_type_mod, only : bnds_type
+    INTEGER, INTENT(IN)   :: nlon, nz, nb  ! Size of the horizontal and vertical
+    REAL, INTENT(INOUT)   :: t(nlon,nz,nb)
+    REAL, INTENT(INOUT)   :: q(nlon,nz,nb)
+    INTEGER :: b, start, end
+    TYPE(bnds_type) :: bnds
+
+    bnds%start = 1
+    bnds%end = nlon
+    do b=1, nb
+      call compute_column(bnds, nlon, nz, q(:,:,b), t(:,:,b))
+    end do
+  END SUBROUTINE column_driver
+"""
+
+    fcode_kernel = """
+  SUBROUTINE compute_column(bnds, nlon, nz, q, t)
+    USE bnds_type_mod, only : bnds_type
+    TYPE(bnds_type), INTENT(IN) :: bnds
+    INTEGER, INTENT(IN) :: nlon, nz    ! Size of the horizontal and vertical
+    REAL, INTENT(INOUT) :: t(nlon,nz)
+    REAL, INTENT(INOUT) :: q(nlon,nz)
+    INTEGER :: jl, jk
+    REAL :: c
+
+    c = 5.345
+    DO jk = 2, nz
+      DO jl = bnds%start, bnds%end
+        t(jl, jk) = c * jk
+        q(jl, jk) = q(jl, jk-1) + t(jl, jk) * c
+      END DO
+    END DO
+
+    ! The scaling is purposefully upper-cased
+    DO JL = BNDS%START, BNDS%END
+      Q(JL, NZ) = Q(JL, NZ) * C
+    END DO
+  END SUBROUTINE compute_column
+"""
+    bnds_type_mod = Sourcefile.from_source(fcode_bnds_type_mod, frontend=frontend)
+    kernel = Sourcefile.from_source(fcode_kernel, frontend=frontend,
+                                    definitions=bnds_type_mod.definitions).subroutines[0]
+    driver = Sourcefile.from_source(fcode_driver, frontend=frontend,
+                                    definitions=bnds_type_mod.definitions).subroutines[0]
+
+    # Ensure we have three loops in the kernel prior to transformation
+    kernel_loops = FindNodes(Loop).visit(kernel.body)
+    assert len(kernel_loops) == 3
+
+    scc_transform = (SCCDevectorTransformation(horizontal=horizontal_bounds_aliases),)
+    scc_transform += (SCCRevectorTransformation(horizontal=horizontal_bounds_aliases),)
+    for transform in scc_transform:
+        transform.apply(driver, role='driver')
+        transform.apply(kernel, role='kernel')
+
+    # Ensure we have two nested loops in the kernel
+    # (the hoisted horizontal and the native vertical)
+    kernel_loops = FindNodes(Loop).visit(kernel.body)
+    assert len(kernel_loops) == 2
+    assert kernel_loops[1] in FindNodes(Loop).visit(kernel_loops[0].body)
+    assert kernel_loops[0].variable == 'jl'
+    assert kernel_loops[0].bounds == 'bnds%start:bnds%end'
+    assert kernel_loops[1].variable == 'jk'
+    assert kernel_loops[1].bounds == '2:nz'
+
+    # Ensure all expressions and array indices are unchanged
+    assigns = FindNodes(Assignment).visit(kernel.body)
+    assert fgen(assigns[1]).lower() == 't(jl, jk) = c*jk'
+    assert fgen(assigns[2]).lower() == 'q(jl, jk) = q(jl, jk - 1) + t(jl, jk)*c'
+    assert fgen(assigns[3]).lower() == 'q(jl, nz) = q(jl, nz)*c'
+
+    # Ensure driver remains unaffected
+    driver_loops = FindNodes(Loop).visit(driver.body)
+    assert len(driver_loops) == 1
+    assert driver_loops[0].variable == 'b'
+    assert driver_loops[0].bounds == '1:nb'
+
+    kernel_calls = FindNodes(CallStatement).visit(driver_loops[0])
+    assert len(kernel_calls) == 1
+    assert kernel_calls[0].name == 'compute_column'
+
+
 @pytest.mark.parametrize('frontend', available_frontends())
 def test_scc_base_resolve_vector_notation(frontend, horizontal):
     """
@@ -235,14 +341,16 @@ def test_scc_demote_transformation(frontend, horizontal):
     """
 
     fcode_kernel = """
-  SUBROUTINE compute_column(start, end, nlon, nz, q)
+  SUBROUTINE compute_column(start, end, nlon, nproma, nz, q)
     INTEGER, INTENT(IN) :: start, end  ! Iteration indices
     INTEGER, INTENT(IN) :: nlon, nz    ! Size of the horizontal and vertical
+    INTEGER, INTENT(IN) :: nproma      ! Horizontal size alias
     REAL, INTENT(INOUT) :: q(nlon,nz)
     REAL :: t(nlon,nz)
-    REAL :: a(nlon)
+    REAL :: a(nproma)
     REAL :: b(nlon,psize)
     REAL :: unused(nlon)
+    REAL :: d(nlon,psize)
     INTEGER, PARAMETER :: psize = 3
     INTEGER :: jl, jk
     REAL :: c
@@ -262,18 +370,24 @@ def test_scc_demote_transformation(frontend, horizontal):
       b(jl, 2) = Q(JL, 3)
       b(jl, 3) = a(jl) * (b(jl, 1) + b(jl, 2))
 
+      d(jl, 1) = b(jl, 1)
+      d(jl, 2) = b(jl, 2)
+      d(jl, 3) = b(jl, 3)
+
       Q(JL, NZ) = Q(JL, NZ) * C + b(jl, 3)
     END DO
   END SUBROUTINE compute_column
 """
-    kernel = Subroutine.from_source(fcode_kernel, frontend=frontend)
+    kernel_source = Sourcefile.from_source(fcode_kernel, frontend=frontend)
+    kernel_item = ProcedureItem(name='#compute_column', source=kernel_source, config={'preserve_arrays': ['d',]})
+    kernel = kernel_source.subroutines[0]
 
     # Must run SCCDevector first because demotion relies on knowledge
     # of vector sections
     scc_transform = (SCCDevectorTransformation(horizontal=horizontal),)
     scc_transform += (SCCDemoteTransformation(horizontal=horizontal),)
     for transform in scc_transform:
-        transform.apply(kernel, role='kernel')
+        transform.apply(kernel, role='kernel', item=kernel_item)
 
     # Ensure correct array variables shapes
     assert isinstance(kernel.variable_map['a'], Scalar)
@@ -282,6 +396,7 @@ def test_scc_demote_transformation(frontend, horizontal):
     assert isinstance(kernel.variable_map['t'], Array)
     assert isinstance(kernel.variable_map['q'], Array)
     assert isinstance(kernel.variable_map['unused'], Scalar)
+    assert isinstance(kernel.variable_map['d'], Array)
 
     # Ensure that parameter-sized array b got demoted only
     assert kernel.variable_map['b'].shape == ((3,) if frontend is OMNI else ('psize',))
@@ -296,11 +411,14 @@ def test_scc_demote_transformation(frontend, horizontal):
     assert fgen(assigns[4]).lower() == 'b(1) = q(jl, 2)'
     assert fgen(assigns[5]).lower() == 'b(2) = q(jl, 3)'
     assert fgen(assigns[6]).lower() == 'b(3) = a*(b(1) + b(2))'
-    assert fgen(assigns[7]).lower() == 'q(jl, nz) = q(jl, nz)*c + b(3)'
+    assert fgen(assigns[7]).lower() == 'd(jl, 1) = b(1)'
+    assert fgen(assigns[8]).lower() == 'd(jl, 2) = b(2)'
+    assert fgen(assigns[9]).lower() == 'd(jl, 3) = b(3)'
+    assert fgen(assigns[10]).lower() == 'q(jl, nz) = q(jl, nz)*c + b(3)'
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_scc_hoist_multiple_kernels(frontend, horizontal, vertical, blocking):
+def test_scc_hoist_multiple_kernels(frontend, horizontal, blocking):
     """
     Test hoisting of column temporaries to "driver" level.
     """
@@ -354,27 +472,16 @@ def test_scc_hoist_multiple_kernels(frontend, horizontal, vertical, blocking):
     driver_item = ProcedureItem(name='#column_driver', source=driver_source)
     kernel_item = ProcedureItem(name='#compute_column', source=kernel_source)
 
-    scc_transform = (SCCDevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCDemoteTransformation(horizontal=horizontal),)
-    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
+    scc_hoist = SCCHoistPipeline(
+        horizontal=horizontal, block_dim=blocking, directive='openacc'
+    )
 
-    for transform in scc_transform:
-        transform.apply(driver, role='driver', item=driver_item, targets=['compute_column'])
-        transform.apply(kernel, role='kernel', item=kernel_item)
-
-    # Now apply the hoisting passes (anaylisis in reverse order)
-    analysis = HoistTemporaryArraysAnalysis(dim_vars=(vertical.size,))
-    synthesis = SCCHoistTemporaryArraysTransformation(block_dim=blocking)
-    analysis.apply(kernel, role='kernel', item=kernel_item)
-    analysis.apply(driver, role='driver', item=driver_item, successors=(kernel_item,))
-    synthesis.apply(driver, role='driver', item=driver_item, successors=(kernel_item,))
-    synthesis.apply(kernel, role='kernel', item=kernel_item)
-
-    annotate = SCCAnnotateTransformation(
-        horizontal=horizontal, vertical=vertical, directive='openacc', block_dim=blocking
+    # Apply pipeline in reverse order to ensure analysis runs before hoisting
+    scc_hoist.apply(kernel, role='kernel', item=kernel_item)
+    scc_hoist.apply(
+        driver, role='driver', item=driver_item,
+        successors=(kernel_item,), targets=['compute_column']
     )
-    annotate.apply(driver, role='driver', item=driver_item, targets=['compute_column'])
-    annotate.apply(kernel, role='kernel', item=kernel_item)
 
     # Ensure we two loops left in kernel
     kernel_loops = FindNodes(Loop).visit(kernel.body)
@@ -413,7 +520,7 @@ def test_scc_hoist_multiple_kernels(frontend, horizontal, vertical, blocking):
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('trim_vector_sections', [True, False])
-def test_scc_hoist_multiple_kernels_loops(frontend, trim_vector_sections, horizontal, vertical, blocking):
+def test_scc_hoist_multiple_kernels_loops(frontend, trim_vector_sections, horizontal, blocking):
     """
     Test hoisting of column temporaries to "driver" level.
     """
@@ -528,7 +635,7 @@ END MODULE kernel_mod
     transformation += (SCCDemoteTransformation(horizontal=horizontal),)
     transformation += (SCCRevectorTransformation(horizontal=horizontal),)
     transformation += (SCCAnnotateTransformation(
-        horizontal=horizontal, vertical=vertical, directive='openacc', block_dim=blocking,
+        horizontal=horizontal, directive='openacc', block_dim=blocking,
     ),)
     for transform in transformation:
         scheduler.process(transformation=transform)
@@ -618,32 +725,36 @@ END MODULE kernel_mod
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_scc_annotate_openacc(frontend, horizontal, vertical, blocking):
+def test_scc_annotate_openacc(frontend, horizontal, blocking):
     """
     Test the correct addition of OpenACC pragmas to SCC format code (no hoisting).
     """
 
     fcode_driver = """
-  SUBROUTINE column_driver(nlon, nz, q, nb)
+  SUBROUTINE column_driver(nlon, nproma, nlev, nz, q, nb)
     INTEGER, INTENT(IN)   :: nlon, nz, nb  ! Size of the horizontal and vertical
+    INTEGER, INTENT(IN)   :: nproma, nlev  ! Aliases of horizontal and vertical sizes
     REAL, INTENT(INOUT)   :: q(nlon,nz,nb)
     INTEGER :: b, start, end
 
     start = 1
     end = nlon
     do b=1, nb
-      call compute_column(start, end, nlon, nz, q(:,:,b))
+      call compute_column(start, end, nlon, nproma, nz, q(:,:,b))
     end do
   END SUBROUTINE column_driver
 """
 
     fcode_kernel = """
-  SUBROUTINE compute_column(start, end, nlon, nz, q)
-    INTEGER, INTENT(IN) :: start, end  ! Iteration indices
-    INTEGER, INTENT(IN) :: nlon, nz    ! Size of the horizontal and vertical
+  SUBROUTINE compute_column(start, end, nlon, nproma, nlev, nz, q)
+    INTEGER, INTENT(IN) :: start, end   ! Iteration indices
+    INTEGER, INTENT(IN) :: nlon, nz     ! Size of the horizontal and vertical
+    INTEGER, INTENT(IN) :: nproma, nlev ! Aliases of horizontal and vertical sizes
     REAL, INTENT(INOUT) :: q(nlon,nz)
     REAL :: t(nlon,nz)
     REAL :: a(nlon)
+    REAL :: d(nproma)
+    REAL :: e(nlev)
     REAL :: b(nlon,psize)
     INTEGER, PARAMETER :: psize = 3
     INTEGER :: jl, jk
@@ -676,8 +787,8 @@ def test_scc_annotate_openacc(frontend, horizontal, vertical, blocking):
     scc_transform = (SCCDevectorTransformation(horizontal=horizontal),)
     scc_transform += (SCCDemoteTransformation(horizontal=horizontal),)
     scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCAnnotateTransformation(horizontal=horizontal, vertical=vertical, directive='openacc',
-                                              block_dim=blocking),)
+    scc_transform += (SCCAnnotateTransformation(horizontal=horizontal,
+                                                directive='openacc', block_dim=blocking),)
     for transform in scc_transform:
         transform.apply(driver, role='driver', targets=['compute_column'])
         transform.apply(kernel, role='kernel')
@@ -716,9 +827,16 @@ def test_single_column_coalesced_hoist_openacc(frontend, horizontal, vertical, b
     when hoisting array temporaries to driver.
     """
 
+    fcode_mod = """
+  MODULE BLOCK_DIM_MOD
+     INTEGER :: nb
+  END MODULE BLOCK_DIM_MOD
+  """
+
     fcode_driver = """
-  SUBROUTINE column_driver(nlon, nz, q, nb)
-    INTEGER, INTENT(IN)   :: nlon, nz, nb  ! Size of the horizontal and vertical
+  SUBROUTINE column_driver(nlon, nz, q)
+    USE BLOCK_DIM_MOD, ONLY : nb
+    INTEGER, INTENT(IN)   :: nlon, nz  ! Size of the horizontal and vertical
     REAL, INTENT(INOUT)   :: q(nlon,nz,nb)
     INTEGER :: b, start, end
 
@@ -770,8 +888,9 @@ end module my_scaling_value_mod
 """.strip()
 
     # Mimic the scheduler internal mechanis to apply the transformation cascade
+    mod_source = Sourcefile.from_source(fcode_mod, frontend=frontend)
     kernel_source = Sourcefile.from_source(fcode_kernel, frontend=frontend)
-    driver_source = Sourcefile.from_source(fcode_driver, frontend=frontend)
+    driver_source = Sourcefile.from_source(fcode_driver, frontend=frontend, definitions=mod_source.modules)
     module_source = Sourcefile.from_source(fcode_module, frontend=frontend)
     driver = driver_source['column_driver']
     kernel = kernel_source['compute_column']
@@ -783,35 +902,19 @@ end module my_scaling_value_mod
     kernel_item = ProcedureItem(name='#compute_column', source=kernel_source)
     module_item = ModuleItem(name='my_scaling_value_mod', source=module_source)
 
-    # Test OpenACC annotations on hoisted version
-    scc_transform = (SCCDevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCDemoteTransformation(horizontal=horizontal),)
-    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
-
-    for transform in scc_transform:
-        transform.apply(driver, role='driver', item=driver_item, targets=['compute_column'], successors=[kernel_item])
-        transform.apply(kernel, role='kernel', item=kernel_item, successors=[module_item])
-
-    # Now apply the hoisting passes (anaylisis in reverse order)
-    analysis = HoistTemporaryArraysAnalysis(dim_vars=(vertical.size,))
-    synthesis = SCCHoistTemporaryArraysTransformation(block_dim=blocking)
-
-    # The try-except is for checking a bug where HoistTemporaryArraysAnalysis would
-    # access a GlobalVarImportItem, which should not happen. Note that in case of a KeyError (which signifies
-    # the issue occurring), an explicit pytest failure is thrown to signify that there is no bug in the test itself.
-    try:
-        analysis.apply(kernel, role='kernel', item=kernel_item, successors=(module_item,))
-    except KeyError:
-        pytest.fail('`HoistTemporaryArraysAnalysis` should not attempt to access `GlobalVarImportItem`s')
-    analysis.apply(driver, role='driver', item=driver_item, successors=(kernel_item,))
-    synthesis.apply(driver, role='driver', item=driver_item, successors=(kernel_item,))
-    synthesis.apply(kernel, role='kernel', item=kernel_item, successors=(module_item,))
-
-    annotate = SCCAnnotateTransformation(
-        horizontal=horizontal, vertical=vertical, directive='openacc', block_dim=blocking
+    scc_hoist = SCCHoistPipeline(
+        horizontal=horizontal, block_dim=blocking,
+        directive='openacc', dim_vars=(vertical.size,)
     )
-    annotate.apply(driver, role='driver', item=driver_item, targets=['compute_column'])
-    annotate.apply(kernel, role='kernel', item=kernel_item)
+
+    # Apply in reverse order to ensure hoisting analysis gets run on kernel first
+    scc_hoist.apply(kernel, role='kernel', item=kernel_item, successors=(module_item,))
+    scc_hoist.apply(
+        driver, role='driver', item=driver_item, successors=(kernel_item,), targets=['compute_column']
+    )
+
+    # Check that blocking size has not been redefined
+    assert driver.symbol_map[blocking.size].type.module.name.lower() == 'block_dim_mod'
 
     with pragmas_attached(kernel, Loop):
         # Ensure kernel routine is anntoated at vector level
@@ -923,34 +1026,21 @@ def test_single_column_coalesced_hoist_nested_openacc(frontend, horizontal, vert
     outer_kernel_item = ProcedureItem(name='#compute_column', source=outer_kernel)
     inner_kernel_item = ProcedureItem(name='#update_q', source=inner_kernel)
 
-    # Test OpenACC annotations on hoisted version
-    scc_transform = (SCCDevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCDemoteTransformation(horizontal=horizontal),)
-    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
+    scc_hoist = SCCHoistPipeline(
+        horizontal=horizontal, block_dim=blocking,
+        dim_vars=(vertical.size,), as_kwarguments=as_kwarguments, directive='openacc'
+    )
 
-    for transform in scc_transform:
-        transform.apply(driver, role='driver', item=driver_item, targets=['compute_column'])
-        transform.apply(outer_kernel, role='kernel', item=outer_kernel_item, targets=['compute_q'])
-        transform.apply(inner_kernel, role='kernel', item=inner_kernel_item)
-
-    # Now apply the hoisting passes (anaylisis in reverse order)
-    analysis = HoistTemporaryArraysAnalysis(dim_vars=(vertical.size,))
-    synthesis = SCCHoistTemporaryArraysTransformation(block_dim=blocking, as_kwarguments=as_kwarguments)
-    # analysis reverse order
-    analysis.apply(inner_kernel, role='kernel', item=inner_kernel_item)
-    analysis.apply(outer_kernel, role='kernel', item=outer_kernel_item, successors=(inner_kernel_item,))
-    analysis.apply(driver, role='driver', item=driver_item, successors=(outer_kernel_item,))
-    # synthesis
-    synthesis.apply(driver, role='driver', item=driver_item, successors=(outer_kernel_item,))
-    synthesis.apply(outer_kernel, role='kernel', item=outer_kernel_item, successors=(inner_kernel_item,))
-    synthesis.apply(inner_kernel, role='kernel', item=outer_kernel_item)
-
-    annotate = SCCAnnotateTransformation(
-        horizontal=horizontal, vertical=vertical, directive='openacc', block_dim=blocking
+    # Apply in reverse order to ensure hoisting analysis gets run on kernel first
+    scc_hoist.apply(inner_kernel, role='kernel', item=inner_kernel_item)
+    scc_hoist.apply(
+        outer_kernel, role='kernel', item=outer_kernel_item,
+        targets=['compute_q'], successors=(inner_kernel_item,)
+    )
+    scc_hoist.apply(
+        driver, role='driver', item=driver_item,
+        targets=['compute_column'], successors=(outer_kernel_item,)
     )
-    annotate.apply(driver, role='driver', item=driver_item, targets=['compute_column'])
-    annotate.apply(outer_kernel, role='kernel', item=outer_kernel_item, targets=['update_q'])
-    annotate.apply(inner_kernel, role='kernel', item=outer_kernel_item)
 
     # Ensure calls have correct arguments
     # driver
@@ -1040,7 +1130,7 @@ def test_single_column_coalesced_hoist_nested_openacc(frontend, horizontal, vert
         assert outer_kernel_pragmas[2].content == 'end data'
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_single_column_coalesced_nested(frontend, horizontal, vertical, blocking):
+def test_single_column_coalesced_nested(frontend, horizontal, blocking):
     """
     Test the correct handling of nested vector-level routines in SCC.
     """
@@ -1106,22 +1196,21 @@ def test_single_column_coalesced_nested(frontend, horizontal, vertical, blocking
     outer_kernel.enrich(inner_kernel)  # Attach kernel source to driver call
     driver.enrich(outer_kernel)  # Attach kernel source to driver call
 
-    # Test SCC transform for plain nested kernel
-    scc_transform = (SCCBaseTransformation(horizontal=horizontal),)
-    scc_transform += (SCCDevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCAnnotateTransformation(horizontal=horizontal, vertical=vertical,
-                                                directive='openacc', block_dim=blocking),)
+    # Instantial SCCVector pipeline and apply
+    scc_pipeline = SCCVectorPipeline(
+        horizontal=horizontal, block_dim=blocking, directive='openacc'
+    )
+    scc_pipeline.apply(driver, role='driver', targets=['compute_column'])
+    scc_pipeline.apply(outer_kernel, role='kernel', targets=['compute_q'])
+    scc_pipeline.apply(inner_kernel, role='kernel')
 
     # Apply annotate twice to test bailing out mechanism
-    scc_transform += (SCCAnnotateTransformation(horizontal=horizontal, vertical=vertical,
-                                                directive='openacc', block_dim=blocking),)
-
-
-    for transform in scc_transform:
-        transform.apply(driver, role='driver', targets=['compute_column'])
-        transform.apply(outer_kernel, role='kernel', targets=['compute_q'])
-        transform.apply(inner_kernel, role='kernel')
+    scc_annotate = SCCAnnotateTransformation(
+        horizontal=horizontal, directive='openacc', block_dim=blocking
+    )
+    scc_annotate.apply(driver, role='driver', targets=['compute_column'])
+    scc_annotate.apply(outer_kernel, role='kernel', targets=['compute_q'])
+    scc_annotate.apply(inner_kernel, role='kernel')
 
     # Ensure a single outer parallel loop in driver
     with pragmas_attached(driver, Loop):
@@ -1191,7 +1280,7 @@ def test_single_column_coalesced_nested(frontend, horizontal, vertical, blocking
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_single_column_coalesced_outer_loop(frontend, horizontal, vertical, blocking):
+def test_single_column_coalesced_outer_loop(frontend, horizontal, blocking):
     """
     Test the correct handling of an outer loop that breaks scoping.
     """
@@ -1241,12 +1330,10 @@ def test_single_column_coalesced_outer_loop(frontend, horizontal, vertical, bloc
     kernel = Subroutine.from_source(fcode_kernel, frontend=frontend)
 
     # Test SCC transform for kernel with scope-splitting outer loop
-    scc_transform = (SCCDevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCAnnotateTransformation(horizontal=horizontal, vertical=vertical,
-                                                directive='openacc', block_dim=blocking),)
-    for transform in scc_transform:
-        transform.apply(kernel, role='kernel')
+    scc_pipeline = SCCVectorPipeline(
+        horizontal=horizontal, block_dim=blocking, directive='openacc'
+    )
+    scc_pipeline.apply(kernel, role='kernel')
 
     # Ensure that we capture vector loops outside the outer vertical loop, as well as the one vector loop inside it.
     with pragmas_attached(kernel, Loop):
@@ -1411,7 +1498,7 @@ def test_single_column_coalesced_variable_demotion(frontend, horizontal):
 
 @pytest.mark.parametrize('frontend', available_frontends(xfail=[(OFP,
                          'OFP fails to parse multiconditional with embedded call.')]))
-def test_single_column_coalesced_multicond(frontend, horizontal, vertical, blocking):
+def test_single_column_coalesced_multicond(frontend, horizontal, blocking):
     """
     Test if horizontal loops in multiconditionals with CallStatements are
     correctly transformed.
@@ -1446,13 +1533,10 @@ def test_single_column_coalesced_multicond(frontend, horizontal, vertical, block
 
     kernel = Subroutine.from_source(fcode, frontend=frontend)
 
-    scc_transform = (SCCBaseTransformation(horizontal=horizontal),)
-    scc_transform += (SCCDevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCAnnotateTransformation(horizontal=horizontal, vertical=vertical,
-                                                directive='openacc', block_dim=blocking),)
-    for transform in scc_transform:
-        transform.apply(kernel, role='kernel')
+    scc_pipeline = SCCVectorPipeline(
+        horizontal=horizontal, block_dim=blocking, directive='openacc'
+    )
+    scc_pipeline.apply(kernel, role='kernel')
 
     # Ensure we have three vector loops in the kernel
     kernel_loops = FindNodes(Loop).visit(kernel.body)
@@ -1476,7 +1560,7 @@ def test_single_column_coalesced_multicond(frontend, horizontal, vertical, block
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_single_column_coalesced_multiple_acc_pragmas(frontend, horizontal, vertical, blocking):
+def test_single_column_coalesced_multiple_acc_pragmas(frontend, horizontal, blocking):
     """
     Test that both '!$acc data' and '!$acc parallel loop gang' pragmas are created at the
     driver layer.
@@ -1521,12 +1605,10 @@ def test_single_column_coalesced_multiple_acc_pragmas(frontend, horizontal, vert
     data_offload = DataOffloadTransformation(remove_openmp=True)
     data_offload.transform_subroutine(routine, role='driver', targets=['some_kernel',])
 
-    scc_transform = (SCCDevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCAnnotateTransformation(horizontal=horizontal, vertical=vertical,
-                                                directive='openacc', block_dim=blocking),)
-    for transform in scc_transform:
-        transform.apply(routine, role='driver', targets=['some_kernel',])
+    scc_pipeline = SCCVectorPipeline(
+        horizontal=horizontal, block_dim=blocking, directive='openacc'
+    )
+    scc_pipeline.apply(routine, role='driver', targets=['some_kernel',])
 
     # Check that both acc pragmas are created
     pragmas = FindNodes(Pragma).visit(routine.ir)
@@ -1638,7 +1720,7 @@ def test_single_column_coalesced_vector_inlined_call(frontend, horizontal):
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_single_column_coalesced_vector_reduction(frontend, horizontal, vertical, blocking):
+def test_single_column_coalesced_vector_reduction(frontend, horizontal, blocking):
     """
     Test for the insertion of OpenACC vector reduction directives.
     """
@@ -1660,10 +1742,9 @@ def test_single_column_coalesced_vector_reduction(frontend, horizontal, vertical
     end subroutine some_kernel
     """
 
-    scc_transform = (SCCDevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
-    scc_transform += (SCCAnnotateTransformation(horizontal=horizontal, vertical=vertical,
-                                                directive='openacc', block_dim=blocking),)
+    scc_pipeline = SCCVectorPipeline(
+        horizontal=horizontal, block_dim=blocking, directive='openacc'
+    )
 
     source = Sourcefile.from_source(fcode, frontend=frontend)
     routine = source['some_kernel']
@@ -1672,8 +1753,8 @@ def test_single_column_coalesced_vector_reduction(frontend, horizontal, vertical
         region = FindNodes(PragmaRegion).visit(routine.body)
         assert is_loki_pragma(region[0].pragma, starts_with = 'vector-reduction')
 
-    for transform in scc_transform:
-        transform.apply(routine, role='kernel', targets=['some_kernel',])
+
+    scc_pipeline.apply(routine, role='kernel', targets=['some_kernel',])
 
     pragmas = FindNodes(Pragma).visit(routine.body)
     assert len(pragmas) == 3
@@ -1736,11 +1817,19 @@ def test_single_column_coalesced_demotion_parameter(frontend, horizontal):
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_scc_base_horizontal_bounds_checks(frontend, horizontal):
+def test_scc_base_horizontal_bounds_checks(frontend, horizontal, horizontal_bounds_aliases):
     """
     Test the SCCBaseTransformation checks for horizontal loop bounds.
     """
 
+    fcode = """
+    subroutine kernel(start, end, work)
+      real, intent(inout) :: work
+      integer, intent(in) :: start, end
+
+    end subroutine kernel
+"""
+
     fcode_no_start = """
     subroutine kernel(end, work)
       real, intent(inout) :: work
@@ -1757,8 +1846,27 @@ def test_scc_base_horizontal_bounds_checks(frontend, horizontal):
     end subroutine kernel
 """
 
+    fcode_alias = """
+    module bnds_type_mod
+    implicit none
+       type bnds_type
+          integer :: start
+          integer :: end
+       end type bnds_type
+    end module bnds_type_mod
+
+    subroutine kernel(bnds, work)
+      use bnds_type_mod, only : bnds_type
+      type(bnds_type), intent(in) :: bnds
+      real, intent(inout) :: work
+
+    end subroutine kernel
+"""
+
+    routine = Subroutine.from_source(fcode, frontend=frontend)
     no_start = Subroutine.from_source(fcode_no_start, frontend=frontend)
     no_end = Subroutine.from_source(fcode_no_end, frontend=frontend)
+    alias = Sourcefile.from_source(fcode_alias, frontend=frontend).subroutines[0]
 
     transform = SCCBaseTransformation(horizontal=horizontal)
     with pytest.raises(RuntimeError):
@@ -1766,6 +1874,16 @@ def test_scc_base_horizontal_bounds_checks(frontend, horizontal):
     with pytest.raises(RuntimeError):
         transform.apply(no_end, role='kernel')
 
+    transform = SCCBaseTransformation(horizontal=horizontal_bounds_aliases)
+    transform.apply(alias, role='kernel')
+
+    bounds = SCCBaseTransformation.get_horizontal_loop_bounds(routine, horizontal_bounds_aliases)
+    assert bounds[0] == 'start'
+    assert bounds[1] == 'end'
+
+    bounds = SCCBaseTransformation.get_horizontal_loop_bounds(alias, horizontal_bounds_aliases)
+    assert bounds[0] == 'bnds%start'
+    assert bounds[1] == 'bnds%end'
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('trim_vector_sections', [False, True])
@@ -1877,7 +1995,9 @@ def test_single_column_coalesced_vector_section_trim_nested(frontend, horizontal
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('trim_vector_sections', [False, True])
-def test_single_column_coalesced_vector_section_trim_complex(frontend, horizontal, trim_vector_sections):
+def test_single_column_coalesced_vector_section_trim_complex(
+        frontend, horizontal, vertical, blocking, trim_vector_sections
+):
     """
     Test to highlight the limitations of vector-section trimming.
     """
@@ -1908,12 +2028,11 @@ def test_single_column_coalesced_vector_section_trim_complex(frontend, horizonta
 
     routine = Subroutine.from_source(fcode_kernel, frontend=frontend)
 
-    scc_transform = (SCCBaseTransformation(horizontal=horizontal),)
-    scc_transform += (SCCDevectorTransformation(horizontal=horizontal, trim_vector_sections=trim_vector_sections),)
-    scc_transform += (SCCRevectorTransformation(horizontal=horizontal),)
-
-    for transform in scc_transform:
-        transform.apply(routine, role='kernel', targets=['some_kernel',])
+    scc_pipeline = SCCVectorPipeline(
+        horizontal=horizontal, vertical=vertical, block_dim=blocking,
+        directive='openacc', trim_vector_sections=trim_vector_sections
+    )
+    scc_pipeline.apply(routine, role='kernel', targets=['some_kernel',])
 
     assign = FindNodes(Assignment).visit(routine.body)[0]
 
