@@ -18,10 +18,11 @@ from loki import (
     CaseInsensitiveDict, as_tuple, flatten, types
 )
 
-from transformations.single_column_coalesced import SCCBaseTransformation
+from transformations.single_column_base import SCCBaseTransformation
 from transformations.single_column_coalesced_vector import SCCDevectorTransformation
 
-__all__ = ['SccCufTransformation', 'HoistTemporaryArraysDeviceAllocatableTransformation']
+__all__ = ['SccCufTransformation', 'HoistTemporaryArraysDeviceAllocatableTransformation',
+        'HoistTemporaryArraysPragmaOffloadTransformation']
 
 
 class HoistTemporaryArraysDeviceAllocatableTransformation(HoistVariablesTransformation):
@@ -60,6 +61,34 @@ class HoistTemporaryArraysDeviceAllocatableTransformation(HoistVariablesTransfor
                 routine.body.append(ir.Deallocation((var.clone(dimensions=None),)))
 
 
+class HoistTemporaryArraysPragmaOffloadTransformation(HoistVariablesTransformation):
+    """
+    Synthesis part for variable/array hoisting, offload via pragmas e.g., OpenACC.
+    """
+
+    def driver_variable_declaration(self, routine, variables):
+        """
+        Standard Variable/Array declaration including
+        device offload via pragmas.
+
+        Parameters
+        ----------
+        routine: :any:`Subroutine`
+            The subroutine to add the variable declaration
+        var: :any:`Variable`
+            The variable to be declared
+        """
+        routine.variables += tuple(var.clone(scope=routine) for var in variables)
+
+        vnames = ', '.join(v.name for v in variables)
+        pragma = ir.Pragma(keyword='acc', content=f'enter data create({vnames})')
+        pragma_post = ir.Pragma(keyword='acc', content=f'exit data delete({vnames})')
+
+        # Add comments around standalone pragmas to avoid false attachment
+        routine.body.prepend((ir.Comment(''), pragma, ir.Comment('')))
+        routine.body.append((ir.Comment(''), pragma_post, ir.Comment('')))
+
+
 def dynamic_local_arrays(routine, vertical):
     """
     Declaring local arrays with the ``vertical`` :any:`Dimension` to be
