@@ -13,7 +13,7 @@ from loki import (
      NestedTransformer, FindVariables, demote_variables, is_dimension_constant,
      is_loki_pragma, dataflow_analysis_attached, BasicType, pragmas_attached
 )
-from transformations.single_column_coalesced import SCCBaseTransformation
+from transformations.single_column_base import SCCBaseTransformation
 
 __all__ = ['SCCDevectorTransformation', 'SCCRevectorTransformation', 'SCCDemoteTransformation']
 
@@ -265,9 +265,13 @@ class SCCRevectorTransformation(Transformation):
             The dimension specifying the horizontal vector dimension
         """
 
+        variable_map = routine.variable_map
+        bounds = SCCBaseTransformation.get_horizontal_loop_bounds(routine, horizontal)
+
+        v_start = routine.resolve_typebound_var(bounds[0], variable_map)
+        v_end = routine.resolve_typebound_var(bounds[1], variable_map)
+
         # Create a single loop around the horizontal from a given body
-        v_start = routine.variable_map[horizontal.bounds[0]]
-        v_end = routine.variable_map[horizontal.bounds[1]]
         index = SCCBaseTransformation.get_integer_variable(routine, horizontal.index)
         bounds = sym.LoopRange((v_start, v_end))
 
@@ -300,6 +304,11 @@ class SCCDemoteTransformation(Transformation):
     A set of utilities to determine which local arrays can be safely demoted in a
     :any:`Subroutine` as part of a transformation pass.
 
+    Unless the option `demote_local_arrays` is set to `False`, this transformation will demote
+    local arrays that do not buffer values between vector loops. Specific arrays in individual
+    routines can also be marked for preservation by assigning them to the `preserve_arrays` list
+    in the :any:`SchedulerConfig`.
+
     Parameters
     ----------
     horizontal : :any:`Dimension`
@@ -326,7 +335,8 @@ class SCCDemoteTransformation(Transformation):
             # Only demote local arrays with the horizontal as fast dimension
             arrays = [v for v in arrays if isinstance(v, sym.Array)]
             arrays = [v for v in arrays if v.name not in argument_names]
-            arrays = [v for v in arrays if v.shape and v.shape[0] == horizontal.size]
+            arrays = [v for v in arrays if v.shape and
+                      v.shape[0] in [horizontal.size, *horizontal._aliases]]
 
             # Also demote arrays whose remaning dimensions are known constants
             arrays = [v for v in arrays if all(is_dimension_constant(d) for d in v.shape[1:])]
@@ -370,11 +380,13 @@ class SCCDemoteTransformation(Transformation):
 
         if role == 'kernel':
             demote_locals = self.demote_local_arrays
+            preserve_arrays = []
             if item:
                 demote_locals = item.config.get('demote_locals', self.demote_local_arrays)
-            self.process_kernel(routine, demote_locals=demote_locals)
+                preserve_arrays = item.config.get('preserve_arrays', [])
+            self.process_kernel(routine, demote_locals=demote_locals, preserve_arrays=preserve_arrays)
 
-    def process_kernel(self, routine, demote_locals=True):
+    def process_kernel(self, routine, demote_locals=True, preserve_arrays=None):
         """
         Applies the SCCDemote utilities to a "kernel" and demotes all suitable local arrays.
 
@@ -392,8 +404,13 @@ class SCCDemoteTransformation(Transformation):
         # may carry buffered values between them, so that we may not demote those!
         to_demote = self.kernel_get_locals_to_demote(routine, sections, self.horizontal)
 
+        # Filter out arrays marked explicitly for preservation
+        if preserve_arrays:
+            to_demote = [v for v in to_demote if not v.name in preserve_arrays]
+
         # Demote all private local variables that do not buffer values between sections
         if demote_locals:
             variables = tuple(v.name for v in to_demote)
             if variables:
-                demote_variables(routine, variable_names=variables, dimensions=self.horizontal.size)
+                demote_variables(routine, variable_names=variables,
+                                 dimensions=[self.horizontal.size, *self.horizontal._aliases])
